import threading
import time
import json

from helpers.constants import MAX_WORKSPACE_DEPLOYMENTS, MAX_WORKSPACE_CREATION_ATTEMPTS, MAX_WORKSPACE_STATUS_VALIDATION_ATTEMPTS
from helpers.deploy import create_app_workspace, get_is_workspace_running


class WorkspacesQueue:
    def __init__(self, max_concurrent_processing=4):
        self.max_concurrent_processing = max_concurrent_processing
        self.items = []
        self.processing_items_count = 0

    def is_empty(self):
        return len(self.items) == 0

    def enqueue(self, item):
        self.items.append(item)
        self.__process_next_item()

    def dequeue(self):
        if not self.is_empty():
            self.processing_items_count += 1
            item = self.items.pop(0)
            try:
                self.__process_item(item)
                self.processing_items_count -= 1
                self.__process_next_item()
            except:
                self.processing_items_count -= 1
                self.enqueue(item)

    def __process_next_item(self):
        if self.processing_items_count < self.max_concurrent_processing:
            thread = threading.Thread(target=self.dequeue)
            thread.start()

    def __process_item(self, item):
        app_name = item.get("app", {}).get("slug")
        print(f"Started the \"{app_name}\" workspace creating.")

        is_workspace_created = False
        create_workspace_attempts = 0

        while not is_workspace_created:
            response = create_app_workspace(item)
            if response.status_code == 200:
                is_workspace_created = True
            else:
                if(create_workspace_attempts > MAX_WORKSPACE_CREATION_ATTEMPTS):
                    reason = json.loads(response.text).get("errors",{})[0].get("message","failed to get error message")
                    break
                create_workspace_attempts += 1
            time.sleep(5)
        
        if not is_workspace_created:
            print(f"\033[31mFailed to create workspace \"{app_name}\". reason: \"{reason}\"\033[0m")
            return

        is_workspace_running = False
        validate_workspace_status_attempts = 0
        while not is_workspace_running:
            if(validate_workspace_status_attempts > MAX_WORKSPACE_STATUS_VALIDATION_ATTEMPTS):
                print(f"\033[31mFailed to validate workspace \"{app_name}\" status\033[0m")
                break
            
            is_workspace_running = get_is_workspace_running(item)
            
            validate_workspace_status_attempts += 1
            time.sleep(5)
        print(f"Finished the \"{app_name}\" workspace creating.")


workspaces_queue = WorkspacesQueue(MAX_WORKSPACE_DEPLOYMENTS)

