import os
import subprocess

from tenacity import retry

from .constants import DE5_APPS_PATH, DE5_GIT_ORIGIN_NAME, RETRY_CONFIG
from .credentials import DE4_HOST, DE4_ADMIN_USERNAME, DE4_ADMIN_PASSWORD, DE5_HOST
from .ssh import get_de5_key_file_name
from .constants import BASE_PATH


@retry(**RETRY_CONFIG)
def clone_app(app_name, use_https=False):

    app_directory = f"{BASE_PATH}/migration/de4/apps/"

    if use_https:
        clone_url = f"https://{DE4_ADMIN_USERNAME}:{DE4_ADMIN_PASSWORD}@{DE4_HOST}/GIT/{app_name}"
        subprocess.call(["git", "-c", "protocol.version=0", "clone", clone_url], cwd=app_directory)
    else:
        ssh_command = f"ssh " \
                    f"-o StrictHostKeyChecking=no " \
                    f"-i {BASE_PATH}/migration/de4/ssh/id_rsa " \
                    f"-F {BASE_PATH}/migration/de4/ssh/ssh_conf"
        clone_url = f"dokku@{DE4_HOST}:{app_name}"
        subprocess.call(["git", "clone", "-c", f"core.sshCommand={ssh_command}", clone_url], cwd=app_directory)

    is_cloned = os.path.exists(f"{app_directory}/{app_name}")
    if not is_cloned:
        print(f"\033[91mThe \"{app_name}\" app cloning failed.\033[0m")
        raise Exception("App is not cloned")


def config_de5_git(app_name, username, folder_subname=""):
    git_url = f"git@{DE5_HOST}:{app_name}"

    subprocess.call(["git", "remote", "add", DE5_GIT_ORIGIN_NAME, git_url], cwd=get_de5_app_path(app_name, folder_subname))
    ssh_command = f"ssh -o \"StrictHostKeyChecking no\" -i {get_de5_key_file_name(username)}"
    subprocess.call(["git", "config", "--add", "--local", "core.sshCommand", ssh_command],
                    cwd=get_de5_app_path(app_name, folder_subname))
    subprocess.call(["git", "config", "--local", "user.name", "'migration-script'"],
                    cwd=get_de5_app_path(app_name, folder_subname))
    subprocess.call(["git", "config", "--local", "user.email", "'migration-script'"],
                    cwd=get_de5_app_path(app_name, folder_subname))
    subprocess.call(["git", "switch", "-c", "main"], cwd=get_de5_app_path(app_name, folder_subname))


def commit_de5_app(app_name, message, folder_subname=""):
    subprocess.call(["git", "add", "."], cwd=get_de5_app_path(app_name, folder_subname))
    subprocess.call(["git", "commit", "-m", message], cwd=get_de5_app_path(app_name, folder_subname))


def push_de5_app(app_name, folder_subname=""):
    subprocess.call(["git", "push", "--force", DE5_GIT_ORIGIN_NAME, "main"], cwd=get_de5_app_path(app_name, folder_subname))


def get_de5_app_path(app_name, folder_subname):
    return f"{DE5_APPS_PATH}/{app_name}{folder_subname}"
