import base64
import os
import sys
from pathlib import Path

import requests
from keycloak import KeycloakOpenID
from requests.structures import CaseInsensitiveDict

from .constants import DE4_WORKSPACES_DATA_DIRECTORY
from .graphql import keycloak_url
from .users import get_de5_user_by_username

sys.path.append(Path(__file__).resolve().parent.parent.__str__())
from .credentials import DE5_HOST


def to_base_64_string(string):
    sample_string_bytes = string.encode("ascii")

    base64_bytes = base64.b64encode(sample_string_bytes)
    return base64_bytes.decode("ascii")


def get_headers(user):
    keycloak_openid = KeycloakOpenID(
        server_url=keycloak_url,
        client_id="dash",
        realm_name="dash"
    )

    token = keycloak_openid.token(user["username"], user["password"], ["password"], code="", redirect_uri="",
                                  totp=None, scope='openid')
    kc_token = to_base_64_string(token['access_token'])
    kc_id_token = to_base_64_string(token['id_token'])

    headers = CaseInsensitiveDict()
    headers["Cookie"] = f'kcToken={kc_token};kcIdToken={kc_id_token}'

    return headers


def get_workspace_directory(app_name):
    return f'{DE4_WORKSPACES_DATA_DIRECTORY}/{app_name}'


def get_workspace_files(app_name):
    res = []
    directory = get_workspace_directory(app_name)
    for (dir_path, dir_names, file_names) in os.walk(directory):
        for file_name in file_names:
            res.append(f"{dir_path}/{file_name}")

    return res


def upload_app_workspace_files(app):
    user = get_de5_user_by_username(app.get("owner"))
    app_name = app.get("app", {}).get("slug")
    files = get_workspace_files(app_name)
    files_count = len(files)
    if files_count > 0:
        print(f"Found {files_count} files for this app workspace. Uploading.")

    for index, file in enumerate(files):
        print(f'Uploading {index + 1}/{files_count}',  end='\r')
        upload_to_de5_workspace(user, app_name, file)


def upload_to_de5_workspace(user, app_name, full_file_path):
    directory = get_workspace_directory(app_name)
    relative_file_path = full_file_path.replace(f"{directory}/", "")

    url = f"https://{DE5_HOST}/workspace/edit/workspace-{app_name}/file-upload"

    headers = get_headers(user)
    files = {
        "uri": (None, f"file:///home/workspaceuser/app/{relative_file_path}"),
        "file": ("File", open(full_file_path, "rb"))
    }

    requests.post(url, headers=headers, files=files)
