import sys
from pathlib import Path
sys.path.append(Path(__file__).resolve().parent.parent.__str__())

import json
from tenacity import retry

from helpers.constants import DE5_APPS_INFO_PATH, DE5_APPS_SLICING, RETRY_CONFIG, DE5_STATUS_FILE_PATH
from helpers.exceptions import handle_app_exception
from helpers.file import get_from_file, save_to_file
from helpers.graphql import de5_request
from helpers.users import get_de5_user_by_username
from queries import GET_DE5_APP_DEPLOYMENTS


def get_apps():
    return get_from_file(DE5_APPS_INFO_PATH)


def safety(value, default):
    return value if value is not None else default


@retry(**RETRY_CONFIG)
def get_app_status(app):
    user = get_de5_user_by_username(app.get("owner"))
    app_name = app.get("app", {}).get("slug")

    graphql = {
        "operationName": "appBySlug",
        "variables": {"slug": app_name},
        "query": GET_DE5_APP_DEPLOYMENTS
    }
    res = de5_request(graphql, user)
    res = json.loads(res.text)
    
    deployments = res.get("data", {}).get('app', {}).get("deployments", [None])
    if not deployments:  # the list is empty
        deployment_status = None  # or some other default value
    else:
        deployment_status = deployments[0]
    
    return {
        app_name: safety(deployment_status, {}.get("deployments", [None])[0])
    }


if __name__ == "__main__":
    apps = get_apps()
    statuses = {}
    for app in apps[DE5_APPS_SLICING]:
        app_name = app.get("app", {}).get("slug")

        def operation():
            status = get_app_status(app)
            app_name = list(status.keys())[0]
            statuses[app_name] = status.get(app_name)

        handle_app_exception(operation, "get status", app_name)

    failed = 0
    success = 0
    failed_apps = []

    for key, value in statuses.items():
        if value is not None and value.get('status', '') == 'running':
            success += 1
        else:
            failed += 1
            failed_apps.append(key)


    print({"failed": failed, "success": success, "failed_apps": failed_apps})

    save_to_file(DE5_STATUS_FILE_PATH, statuses)
    print(f"Saved to {DE5_STATUS_FILE_PATH}")
