import sys
from pathlib import Path
sys.path.append(Path(__file__).resolve().parent.parent.__str__())

from keycloak import KeycloakAdmin
from helpers.users import get_de5_users
from helpers.credentials import DE5_KEYCLOAK_ADMIN_PASSWORD, DE5_KEYCLOAK_ADMIN_USERNAME
from helpers.graphql import keycloak_url


def create_keycloak_admin_client():
    try:
        return KeycloakAdmin(
            server_url=keycloak_url,
            username=DE5_KEYCLOAK_ADMIN_USERNAME,
            password=DE5_KEYCLOAK_ADMIN_PASSWORD,
            realm_name='dash',
            user_realm_name='master',
            verify=True
        )
    except Exception as exc:
        print('\033[91mCant create keycloak client, reason: {}\033[0m'.format(exc))
    exit()


def set_temporary_password_for_user(keycloak_admin, user):
    try:
        user_id = keycloak_admin.get_user_id(user["username"])
        keycloak_admin.set_user_password(user_id, user["password"], temporary=True)
        print(f"Password for user {user['username']} was set.")
    except Exception as e:
        print(
            f"\033[91mCan't set password for this user: {user['username']}."
            f"Are you sure you enabled user modification in the user federation config?"
            f" You can see the reason under this message.\033[0m")
        print(e)


def main():
    users = get_de5_users()

    keycloak_admin_client = create_keycloak_admin_client()

    if users:
        [set_temporary_password_for_user(keycloak_admin_client, user) for user in users]
    else:
        print("Users wasn't handled, reason: got empty array")


if __name__ == "__main__":
    main()
