from gql import gql

GET_APP_NAMES = gql(
    """
    query getApps($page: Int) {
      apps(page:$page, allApps:true) {
        apps {
          name
        }
        nextPage
      }
    }
    """
)

GET_APP_INFO = gql(
    """
    query getApp($name: String) {
      apps(name: $name, allApps: true) {
        apps {
          name
          owner {
            username
          }
          urlOnServer
          thumbnailUrl
          collaborators {
            users
            teams
          }
          metadata {
            title
            description
            tags
            permissionLevel
            showInPortal
            sortRank
          }
          hasWorkspace
          mounts {
            hostDir
            targetDir
            status
          }
          processes {
            type
            number
            status
          }
          analytics {
            appname
          }
          environmentVariables {
            name
            value
            status
            readonly
          }
          linkedServices {
            name
            serviceType
            created
            status
          }
          status {
            running
            deploying
            canScale
          }
          resources {
            type
            status
          }
        }
      }
    }
    """
)

GET_PORTAL_INFO = gql(
    """
    query GetPortal($name: String = "default") {
      portals(name: $name) {
        portals {
          id
          name
          logoUrl
          metadata
          requireLogin
          __typename
        }
        __typename
      }
    }
    """
)

ATTACH_SSH = gql(
    """
    mutation UpdateSshKeys($keys: String!) {
      updateSshKeys(keys: $keys) {
        ok
        error
        __typename
      }
    }
    """
)

UPDATE_APP = """
    mutation updateApp($input: AppUpdate!) {
      updateOneApp(input: $input) {
        app_id
        title
      }
    }
"""

CREATE_SERVICE = """
mutation createService($input: ServiceInput!) {
    createOneService(input: $input) {
        name
        type
        id: service_id
    }
}
"""

ATTACH_DE5_SSH = """
    mutation createOneSshKey($input: sshKeyInput!) {
      createOneSshKey(input: $input) {
          description
          ssh_key_id
          __typename
      }
    }
"""

GET_DE5_APP_DEPLOYMENTS = """
    query appBySlug($slug: String!) {
        app(slug: $slug) {
          deployments(sorting: {field: created_at, direction: DESC}) {
            status
            updated_at
        }
      }
    }
"""

GET_DE5_APP_BUILDS = """
    query appBySlug($slug: String!) {
        app(slug: $slug) {
          app_id
          builds(sorting: {field: created_at, direction: DESC}) {
            git_revision
            status
            started_at
        }
      }
    }
"""


CREATE_DE5_WORKSPACE = """
mutation createOneAppDeployment($input: CreateOneAppDeploymentInput!) {
  createOneAppDeployment(input: $input) {
    workspace
    memory_limit
    cpu_limit
    app_id
    app_deployment_id
    __typename
  }
}
"""

GET_DE5_WORKSPACE_DEPLOYMENTS = """
query appBySlug($slug: String!) {
  app(slug: $slug) {
    workspaces: deployments(filter: {workspace: {is: true}}) {
      status
    }
  }
}
"""


GET_DE4_SSH_KEYS = gql(
    """
    query SshKeys {
        sshKeys {
            keys
        }
    }
"""
)

GET_APP_DEPENDENCY = gql(
    """
query AppOverviewDependencies($name: String!) {
  apps(name: $name, allApps: true) {
    apps {
      name
      analytics {
        dependencies {
          python
        }
      }
    }
  }
}
"""
)
