import csv

import requests

from helpers.constants import DE5_APPS_INFO_PATH, DE5_DETAILED_REPORT_FILE_PATH, DE5_REPORT_FILE_PATH
from helpers.credentials import DE5_HOST
from helpers.file import get_from_file, save_to_file


def get_apps():
    return get_from_file(DE5_APPS_INFO_PATH)


def check_app(app):
    app_name = app.get("app", {}).get("slug")
    url = f"https://{DE5_HOST}/{app_name}"
    result = requests.get(url)
    manager_url = f"https://{DE5_HOST}/apps/{app_name}"

    return {
        "name": app_name,
        "url": url,
        "manager_url": manager_url,
        "status": result.status_code,
        "is_available": str(result.status_code).startswith('2')
    }


if __name__ == "__main__":
    apps = get_apps()
    statuses = []
    report = {
        "statuses": {},
        "availability": {
            "available": 0,
            "not_available": 0,
        }
    }
    for app in apps:
        status = check_app(app)
        report["statuses"][status["status"]] = report["statuses"].get(status["status"], 0) + 1
        if status["is_available"]:
            report["availability"]["available"] += 1
        else:
            report["availability"]["not_available"] += 1
        statuses.append(status)

    save_to_file(DE5_REPORT_FILE_PATH, report)

    to_csv = statuses

    keys = to_csv[0].keys()

    with open(DE5_DETAILED_REPORT_FILE_PATH, 'w', newline='') as output_file:
        dict_writer = csv.DictWriter(output_file, keys)
        dict_writer.writeheader()
        dict_writer.writerows(to_csv)


