import json

from helpers.graphql import de5_request
from helpers.users import get_de5_user_by_username


def get_all_user_apps(user):
    graphql = {"operationName": "myApps",
               "query": """
                    query myApps {
                        myApps(paging: {offset: 0, limit: 150}) {
                            totalCount
                            pageInfo {
                                hasNextPage
                                hasPreviousPage
                                __typename
                            }
                            nodes {
                            id: app_id
                        }
                        __typename
                    }}"""
               }

    res = de5_request(graphql, user)
    return json.loads(res.text).get("data", {}).get("myApps", {}).get("nodes", [])


def delete_app(user, app_id):
    graphql = {"operationName": "deleteApp",
               "variables": {
                   "input": {
                       "id": app_id
                   }
               },
               "query": """
                    mutation deleteApp($input: DeleteOneAppInput!) {
                        deleteOneApp(input: $input) {
                            app_id
                            __typename
                        }
                    }
                    """
               }

    res = de5_request(graphql, user)
    print(res.text)


def delete_apps(user, apps):
    for app in apps:
        delete_app(user, app.get("id"))


if __name__ == "__main__":
    username = input("Enter username: ")
    user = get_de5_user_by_username(username)

    apps = get_all_user_apps(user)

    delete_apps(user, apps)
